@@ -40,6 +40,12 @@ RUN apt-get update && export DEBIAN_FRONTEND=noninteractive \\
         ripgrep fd-find bat \\
     && apt-get clean && rm -rf /var/lib/apt/lists/*
 
+# Install Docker CLI (not daemon) for Docker-in-Docker support
+RUN curl -fsSL https://get.docker.com -o get-docker.sh \\
+    && sh get-docker.sh \\
+    && rm get-docker.sh \\
+    && apt-get clean && rm -rf /var/lib/apt/lists/*
+
 # Configure locale
 RUN sed -i '/en_US.UTF-8/s/^# //g' /etc/locale.gen && locale-gen
 ENV LANG=en_US.UTF-8 LANGUAGE=en_US:en LC_ALL=en_US.UTF-8
@@ -53,11 +59,12 @@ RUN curl -fsSL https://deb.nodesource.com/setup_20.x | bash - \\
 RUN curl -LsSf https://astral.sh/uv/install.sh | sh
 ENV PATH="/root/.local/bin:${PATH}"
 
-# Create non-root user with sudo access
+# Create non-root user with sudo access and add to docker group
 RUN groupadd --gid $USER_GID $USERNAME \\
     && useradd --uid $USER_UID --gid $USER_GID -m $USERNAME -s /bin/zsh \\
     && echo $USERNAME ALL=\\(root\\) NOPASSWD:ALL > /etc/sudoers.d/$USERNAME \\
-    && chmod 0440 /etc/sudoers.d/$USERNAME
+    && chmod 0440 /etc/sudoers.d/$USERNAME \\
+    && usermod -aG docker $USERNAME
 
 # Install Claude Code CLI (version 1.0.60 for stability)
 RUN npm install -g @anthropic-ai/claude-code@1.0.60 \\
@@ -83,6 +90,7 @@ RUN chown -R cuti:cuti /home/cuti
 # Install oh-my-zsh with simple configuration
 RUN sh -c "$(wget -O- https://raw.githubusercontent.com/ohmyzsh/ohmyzsh/master/tools/install.sh)" "" --unattended \\
     && echo 'export PATH="/usr/local/bin:/home/cuti/.local/bin:/root/.local/share/uv/tools/cuti/bin:$PATH"' >> ~/.zshrc \\
+    && echo 'export PYTHONPATH="/workspace/src:$PYTHONPATH"' >> ~/.zshrc \\
     && echo 'export CUTI_IN_CONTAINER=true' >> ~/.zshrc \\
     && echo 'export ANTHROPIC_CLAUDE_BYPASS_PERMISSIONS=1' >> ~/.zshrc \\
     && echo 'export CLAUDE_CONFIG_DIR=/home/cuti/.claude' >> ~/.zshrc \\
@@ -253,6 +261,7 @@ RUN cd /workspace \\
     && python -c "import cuti; print('✅ cuti installed from source')" \\
     && echo '#!/usr/local/bin/python' > /usr/local/bin/cuti \\
     && echo 'import sys' >> /usr/local/bin/cuti \\
+    && echo 'sys.path.insert(0, "/workspace/src")  # Ensure local source takes precedence' >> /usr/local/bin/cuti \\
     && echo 'from cuti.cli.app import app' >> /usr/local/bin/cuti \\
     && echo 'if __name__ == "__main__":' >> /usr/local/bin/cuti \\
     && echo '    app()' >> /usr/local/bin/cuti \\
@@ -448,21 +457,13 @@ RUN /root/.local/bin/uv pip install --system cuti \\
                 print("❌ Couldn't start container runtime")
                 return 1
         
-        # Check for existing .claude directory in workspace
-        workspace_claude = Path.cwd() / ".claude"
-        if workspace_claude.exists() and workspace_claude.is_dir():
-            print("⚠️  Found existing .claude directory in workspace")
-            print("   This may cause permission issues in the container.")
-            print("   ")
-            print("   Options:")
-            print("   1. Remove it: rm -rf .claude")
-            print("   2. Move it: mv .claude .claude.backup")
-            print("   3. Continue anyway (may encounter errors)")
+        # Check Docker Desktop file sharing settings on macOS
+        if self.is_macos:
+            print("📝 Note: If workspace is read-only, check Docker Desktop settings:")
+            print("   1. Open Docker Desktop → Settings → Resources → File Sharing")
+            print("   2. Ensure your project directory is in the shared paths")
+            print("   3. Try 'osxfs' or 'VirtioFS' file sharing implementation")
             print("")
-            response = input("Do you want to continue? (y/N): ")
-            if response.lower() not in ['y', 'yes']:
-                print("Aborted.")
-                return 1
         
         # Build container if needed
         image_name = "cuti-dev-universal"
@@ -476,18 +477,33 @@ RUN /root/.local/bin/uv pip install --system cuti \\
         print("🚀 Starting container...")
         current_dir = Path.cwd().resolve()
         
+        # Try different mount options based on Docker runtime
+        # Colima typically handles mounts better than Docker Desktop on macOS
+        mount_options = "rw"  # Start with basic read-write
+        if self.is_macos:
+            # Check if using Colima (which typically works better with mounts)
+            colima_status = self._run_command(["colima", "status"])
+            if colima_status.returncode == 0 and "running" in colima_status.stdout.lower():
+                print("🐳 Using Colima runtime")
+                mount_options = "rw"  # Colima usually handles basic rw well
+            else:
+                print("🐳 Using Docker Desktop - trying cached mode for better macOS compatibility")
+                mount_options = "rw,cached"  # Docker Desktop on macOS needs cached mode
+        
         docker_args = [
             "docker", "run", "--rm", "--privileged", 
-            "-v", f"{current_dir}:/workspace:rw",  # Explicitly mount workspace as read-write
+            "-v", f"{current_dir}:/workspace:{mount_options}",  # Dynamic mount options
             "-v", f"{Path.home() / '.cuti'}:/root/.cuti-global", 
             "-v", f"{container_claude_dir}:/host-claude-config:ro",  # Mount host's container claude dir as read-only
+            "-v", "/var/run/docker.sock:/var/run/docker.sock",  # Mount Docker socket for Docker-in-Docker
             "-w", "/workspace",
             "--env", "CUTI_IN_CONTAINER=true",
-            "--env", "CLAUDE_QUEUE_STORAGE_DIR=/home/cuti/.cuti",  # Use writable home directory for cuti storage
+            # Don't set CLAUDE_QUEUE_STORAGE_DIR here - let the init script decide based on writability
             "--env", "IS_SANDBOX=1", 
             "--env", "CLAUDE_DANGEROUSLY_SKIP_PERMISSIONS=true",
-            "--env", "CLAUDE_CONFIG_DIR=/home/cuti/.claude",
+            # Don't set CLAUDE_CONFIG_DIR here - let the init script decide based on writability
             "--env", "PYTHONUNBUFFERED=1",
+            "--env", "PYTHONPATH=/workspace/src",
             "--env", "TERM=xterm-256color",
             "--env", "PATH=/usr/local/bin:/home/cuti/.local/bin:/usr/local/sbin:/usr/sbin:/usr/bin:/sbin:/bin",
             "--network", "host",
@@ -500,9 +516,27 @@ RUN /root/.local/bin/uv pip install --system cuti \\
 if touch /workspace/.test_write 2>/dev/null; then
     rm /workspace/.test_write
     WORKSPACE_WRITABLE=true
+    echo "✅ Workspace is writable - Claude can edit code!"
+    # Use workspace directories when writable
+    export CLAUDE_QUEUE_STORAGE_DIR=/workspace/.cuti
+    export CLAUDE_CONFIG_DIR=/workspace/.claude
 else
     WORKSPACE_WRITABLE=false
-    echo "⚠️  Workspace mounted as read-only"
+    echo "⚠️  WARNING: Workspace mounted as read-only!"
+    echo "    This prevents Claude from editing your code."
+    echo ""
+    echo "    To fix this on macOS:"
+    echo "    1. If using Docker Desktop:"
+    echo "       - Go to Settings → Resources → File Sharing"
+    echo "       - Add your project directory to shared folders"
+    echo "       - Switch to 'VirtioFS' under Settings → General"
+    echo "    2. Or use Colima instead (recommended):"
+    echo "       - brew install colima"
+    echo "       - colima start --mount-type 9p"
+    echo ""
+    # Fall back to home directories when read-only
+    export CLAUDE_QUEUE_STORAGE_DIR=/home/cuti/.cuti
+    export CLAUDE_CONFIG_DIR=/home/cuti/.claude
 fi
 
 # Copy mounted claude config to a writable location if needed
@@ -526,19 +560,28 @@ if [ -d /host-claude-config ]; then
     mkdir -p /home/cuti/.claude/ide 2>/dev/null || true
 fi
 
-# Handle workspace .claude directory
-if [ -d /workspace/.claude ]; then
-    if [ "$WORKSPACE_WRITABLE" = "true" ]; then
-        # If writable, ensure proper permissions
-        sudo chown -R cuti:cuti /workspace/.claude 2>/dev/null || true
-        echo "✅ Workspace .claude directory permissions updated"
-    else
-        # If read-only, copy to home and use that
-        echo "📁 Copying workspace .claude to container (workspace is read-only)..."
+# Handle workspace directories based on writability
+if [ "$WORKSPACE_WRITABLE" = "true" ]; then
+    # Create workspace directories if they don't exist
+    mkdir -p /workspace/.claude 2>/dev/null || true
+    mkdir -p /workspace/.cuti 2>/dev/null || true
+    
+    # Ensure proper ownership for workspace directories
+    sudo chown -R cuti:cuti /workspace/.claude 2>/dev/null || true
+    sudo chown -R cuti:cuti /workspace/.cuti 2>/dev/null || true
+    
+    # Copy credentials from host config if not present in workspace
+    if [ ! -f /workspace/.claude/.credentials.json ] && [ -f /home/cuti/.claude/.credentials.json ]; then
+        cp /home/cuti/.claude/.credentials.json /workspace/.claude/.credentials.json 2>/dev/null || true
+    fi
+    
+    echo "📁 Using workspace directories for Claude and cuti"
+else
+    # If read-only, copy workspace config to home if exists
+    if [ -d /workspace/.claude ]; then
+        echo "📁 Copying workspace .claude to home (workspace is read-only)..."
         cp -rn /workspace/.claude/* /home/cuti/.claude/ 2>/dev/null || true
         sudo chown -R cuti:cuti /home/cuti/.claude 2>/dev/null || true
-        # Override CLAUDE_CONFIG_DIR to use home directory
-        export CLAUDE_CONFIG_DIR=/home/cuti/.claude
     fi
 fi
 
@@ -547,24 +590,43 @@ if [ -f /workspace/CLAUDE.md ] && [ ! -f /home/cuti/.claude/CLAUDE.md ]; then
     cp /workspace/CLAUDE.md /home/cuti/.claude/CLAUDE.md 2>/dev/null || true
 fi
 
-# Check for existing credentials
-if [ -f /home/cuti/.claude/.credentials.json ]; then
-    echo "🔑 Found saved Claude credentials - no login needed!"
-elif [ -f /workspace/.claude/.credentials.json ]; then
-    cp /workspace/.claude/.credentials.json /home/cuti/.claude/.credentials.json 2>/dev/null || true
-    echo "🔑 Copied credentials from workspace"
+# Check for existing credentials in the appropriate location
+if [ "$WORKSPACE_WRITABLE" = "true" ]; then
+    if [ -f /workspace/.claude/.credentials.json ]; then
+        echo "🔑 Found saved Claude credentials in workspace - no login needed!"
+    elif [ -f /home/cuti/.claude/.credentials.json ]; then
+        cp /home/cuti/.claude/.credentials.json /workspace/.claude/.credentials.json 2>/dev/null || true
+        echo "🔑 Copied credentials to workspace"
+    else
+        echo "⚠️  No saved credentials. Authenticate once with: claude login"
+        echo "   Your credentials will be saved in the workspace."
+    fi
 else
-    echo "⚠️  No saved credentials. Authenticate once with: claude login"
-    echo "   Your credentials will be saved for all future containers."
+    if [ -f /home/cuti/.claude/.credentials.json ]; then
+        echo "🔑 Found saved Claude credentials - no login needed!"
+    elif [ -f /workspace/.claude/.credentials.json ]; then
+        cp /workspace/.claude/.credentials.json /home/cuti/.claude/.credentials.json 2>/dev/null || true
+        echo "🔑 Copied credentials from workspace"
+    else
+        echo "⚠️  No saved credentials. Authenticate once with: claude login"
+        echo "   Your credentials will be saved for all future containers."
+    fi
+    # Ensure home storage directory exists for read-only mode
+    mkdir -p /home/cuti/.cuti 2>/dev/null || true
 fi
 
-# Ensure cuti storage directory exists
-mkdir -p /home/cuti/.cuti 2>/dev/null || true
+# Ensure PYTHONPATH includes workspace source for local development
+export PYTHONPATH="/workspace/src:$PYTHONPATH"
+echo "🐍 Python path: $PYTHONPATH"
 
-# If workspace is read-only, always use home directory for Claude
-if [ "$WORKSPACE_WRITABLE" = "false" ]; then
-    export CLAUDE_CONFIG_DIR=/home/cuti/.claude
-    echo "   Using /home/cuti/.claude for Claude configuration"
+# Setup Docker socket permissions for Docker-in-Docker
+if [ -S /var/run/docker.sock ]; then
+    echo "🐳 Setting up Docker-in-Docker access..."
+    sudo chgrp docker /var/run/docker.sock 2>/dev/null || true
+    sudo chmod 660 /var/run/docker.sock 2>/dev/null || true
+    echo "✅ Docker socket configured - you can use Docker commands!"
+else
+    echo "⚠️  Docker socket not found - Docker commands won't work in container"
 fi
 
 # Function to sync credentials back to host on exit
